package assignment2;

import java.util.*;
import javax.security.auth.*;
import javax.security.auth.spi.*;
import javax.security.auth.login.*;
import javax.security.auth.callback.*;

public class MyLoginModule implements LoginModule {

	// initial state
	private Subject subject;

	private CallbackHandler callbackHandler;

	private Map sharedState;

	private Map options;

	// username and password
	private String username;

	private String password;

	private String _loginname = " ";

	// authentication status
	boolean auth_success = true;

	boolean commit_success = false;

	boolean password_mismatch = false;

	boolean invalid_user = false;

	private MyPrincipal myPrincipal = null;

	// User Credentials

	public void initialize(Subject subject, CallbackHandler callbackHandler,
			Map sharedState, Map options) {

		System.out.println("MyLoginModule:InitMethod");

		this.subject = subject;
		this.callbackHandler = callbackHandler;
		this.sharedState = sharedState;
		this.options = options;

	}

	public boolean login() throws LoginException {

		System.out.println("MyLoginModule:login()");
		// get the callback handler with the user name and password
		if (callbackHandler == null)
			throw new LoginException(
					"MyLoginModule: No CallbackHandler Available");

		Callback[] callbacks = new Callback[2];
		callbacks[0] = new NameCallback("Username");
		callbacks[1] = new PasswordCallback("Password: ", false);

		try {
			callbackHandler.handle(callbacks);
			username = ((NameCallback) callbacks[0]).getName();
			password = new String(((PasswordCallback) callbacks[1])
					.getPassword());

			auth_success = validateUser(username, password);

			if (!auth_success) {
				if (password_mismatch) {
					throw new LoginException("Invalid Password");
				} else if (invalid_user) {
					throw new LoginException("Invalid Username");
				}
			}
			return true;
		} catch (java.io.IOException ioe) {
			throw new LoginException(ioe.toString());
		} catch (UnsupportedCallbackException use) {
			throw new LoginException("MyLoginModule: Not Supported"
					+ use.getCallback().toString());
		}
	}

	public boolean commit() throws LoginException {

		System.out.println("MyLoginModule:commit()");
		if (auth_success) {
			commit_success = true;
			myPrincipal = new MyPrincipal(_loginname);
			System.out.println("Adding principals");
			subject.getPrincipals().add(myPrincipal);
		} else {
			commit_success = false;
		}

		return commit_success;
	}

	public boolean abort() throws LoginException {

		System.out.println("MyLoginModule:abort()");

		if (!auth_success) {
			// authentication failure
			username = null;
			password = null;
			this.subject.getPrincipals().clear();
			return true;

		}

		return false;
	}

	public boolean logout() throws LoginException {

		System.out.println("MyLoginModule:logout()");
		this.username = null;
		this.password = null;
		this.subject.getPrincipals().clear();
		return true;
	}

	public boolean validateUser(String username, String password) {
		String valid_user = "Admin";
		String valid_pass = "foo123";
		if (username.compareTo(valid_user) != 0
				|| password.compareTo(valid_pass) != 0) {
//		if (password == null || password.compareTo("") == 0) {
			System.err.println("Authentication error");
			return false;
		} else {
			this._loginname = username;
			return true;
		}
	}
}